@@ -61,6 +61,10 @@
 
 #define MAX_SEID 0x3E
 
+#ifndef MAX
+# define MAX(x, y) ((x) > (y) ? (x) : (y))
+#endif
+
 #define AVDTP_DISCOVER				0x01
 #define AVDTP_GET_CAPABILITIES			0x02
 #define AVDTP_SET_CONFIGURATION			0x03
@@ -85,7 +89,7 @@
 #define AVDTP_MSG_TYPE_ACCEPT			0x02
 #define AVDTP_MSG_TYPE_REJECT			0x03
 
-#define REQ_TIMEOUT 4
+#define REQ_TIMEOUT 6
 #define ABORT_TIMEOUT 2
 #define DISCONNECT_TIMEOUT 1
 #define STREAM_TIMEOUT 20
@@ -453,10 +457,8 @@ static void auth_cb(DBusError *derr, void *user_data);
 
 static struct avdtp_server *find_server(GSList *list, const bdaddr_t *src)
 {
-	GSList *l;
-
-	for (l = list; l; l = l->next) {
-		struct avdtp_server *server = l->data;
+	for (; list; list = list->next) {
+		struct avdtp_server *server = list->data;
 
 		if (bacmp(&server->src, src) == 0)
 			return server;
@@ -877,6 +879,8 @@ static void handle_transport_connect(struct avdtp *session, GIOChannel *io,
 {
 	struct avdtp_stream *stream = session->pending_open;
 	struct avdtp_local_sep *sep = stream->lsep;
+	int sk, buf_size, min_buf_size;
+	GError *err = NULL;
 
 	session->pending_open = NULL;
 
@@ -901,22 +905,30 @@ static void handle_transport_connect(struct avdtp *session, GIOChannel *io,
 	stream->omtu = omtu;
 	stream->imtu = imtu;
 
-	/* only if local SEP is of type SRC */
-	if (sep->info.type == AVDTP_SEP_TYPE_SOURCE) {
-		int sk, buf_size, min_buf_size;
+	/* Apply special settings only if local SEP is of type SRC */
+	if (sep->info.type != AVDTP_SEP_TYPE_SOURCE)
+		goto proceed;
 
-		sk = g_io_channel_unix_get_fd(stream->io);
-		buf_size = get_send_buffer_size(sk);
-		if (buf_size < 0)
-			goto proceed;
+	bt_io_set(stream->io, BT_IO_L2CAP, &err,
+					BT_IO_OPT_FLUSHABLE, TRUE,
+					BT_IO_OPT_INVALID);
+	if (err != NULL) {
+		error("Enabling flushable packets failed: %s", err->message);
+		g_error_free(err);
+	} else
+		DBG("Flushable packets enabled");
 
-		DBG("sk %d, omtu %d, send buffer size %d", sk, omtu, buf_size);
-		min_buf_size = omtu * 2;
-		if (buf_size < min_buf_size) {
-			DBG("send buffer size to be increassed to %d",
-								min_buf_size);
-			set_send_buffer_size(sk, min_buf_size);
-		}
+	sk = g_io_channel_unix_get_fd(stream->io);
+	buf_size = get_send_buffer_size(sk);
+	if (buf_size < 0)
+		goto proceed;
+
+	DBG("sk %d, omtu %d, send buffer size %d", sk, omtu, buf_size);
+	min_buf_size = omtu * 2;
+	if (buf_size < min_buf_size) {
+		DBG("send buffer size to be increassed to %d",
+				min_buf_size);
+		set_send_buffer_size(sk, min_buf_size);
 	}
 
 proceed:
@@ -1050,11 +1062,6 @@ static void avdtp_sep_set_state(struct avdtp *session,
 	old_state = sep->state;
 	sep->state = state;
 
-	for (l = stream->callbacks; l != NULL; l = g_slist_next(l)) {
-		struct stream_callback *cb = l->data;
-		cb->cb(stream, old_state, state, err_ptr, cb->user_data);
-	}
-
 	switch (state) {
 	case AVDTP_STATE_CONFIGURED:
 		if (sep->info.type == AVDTP_SEP_TYPE_SINK)
@@ -1086,11 +1093,18 @@ static void avdtp_sep_set_state(struct avdtp *session,
 			handle_unanswered_req(session, stream);
 		/* Remove pending commands for this stream from the queue */
 		cleanup_queue(session, stream);
-		stream_free(stream);
 		break;
 	default:
 		break;
 	}
+
+	for (l = stream->callbacks; l != NULL; l = g_slist_next(l)) {
+		struct stream_callback *cb = l->data;
+		cb->cb(stream, old_state, state, err_ptr, cb->user_data);
+	}
+
+	if (state == AVDTP_STATE_IDLE)
+		stream_free(stream);
 }
 
 static void finalize_discovery(struct avdtp *session, int err)
@@ -2205,10 +2219,8 @@ failed:
 
 static struct avdtp *find_session(GSList *list, const bdaddr_t *dst)
 {
-	GSList *l;
-
-	for (l = list; l != NULL; l = g_slist_next(l)) {
-		struct avdtp *s = l->data;
+	for (; list != NULL; list = g_slist_next(list)) {
+		struct avdtp *s = list->data;
 
 		if (bacmp(dst, &s->dst))
 			continue;
@@ -2340,7 +2352,7 @@ static void avdtp_connect_cb(GIOChannel *chan, GError *err, gpointer user_data)
 	if (session->state == AVDTP_SESSION_STATE_CONNECTING) {
 		DBG("AVDTP imtu=%u, omtu=%u", session->imtu, session->omtu);
 
-		session->buf = g_malloc0(session->imtu);
+		session->buf = g_malloc0(MAX(session->imtu, session->omtu));
 		avdtp_set_state(session, AVDTP_SESSION_STATE_CONNECTED);
 
 		if (session->io_id)
@@ -2644,8 +2656,10 @@ static int send_req(struct avdtp *session, gboolean priority,
 
 	if (session->state == AVDTP_SESSION_STATE_DISCONNECTED) {
 		session->io = l2cap_connect(session);
-		if (!session->io)
+		if (!session->io) {
+			err = -EIO;
 			goto failed;
+		}
 		avdtp_set_state(session, AVDTP_SESSION_STATE_CONNECTING);
 	}
 
@@ -3177,10 +3191,8 @@ gboolean avdtp_stream_has_capability(struct avdtp_stream *stream,
 gboolean avdtp_stream_has_capabilities(struct avdtp_stream *stream,
 					GSList *caps)
 {
-	GSList *l;
-
-	for (l = caps; l; l = g_slist_next(l)) {
-		struct avdtp_service_capability *cap = l->data;
+	for (; caps; caps = g_slist_next(caps)) {
+		struct avdtp_service_capability *cap = caps->data;
 
 		if (!avdtp_stream_has_capability(stream, cap))
 			return FALSE;
@@ -3849,9 +3861,15 @@ void avdtp_exit(const bdaddr_t *src)
 	if (!server)
 		return;
 
-	for (l = server->sessions; l; l = l->next) {
+	l = server->sessions;
+	while (l) {
 		struct avdtp *session = l->data;
 
+		l = l->next;
+		/* value of l pointer should be updated before invoking
+		 * connection_lost since it internally uses avdtp_unref
+		 * which operates on server->session list as well
+		 */
 		connection_lost(session, -ECONNABORTED);
 	}
 
