@@ -37,7 +37,7 @@
 #include <mcap.h>
 #include <btio.h>
 #include <mcap_lib.h>
-#include <l2cap.h>
+#include <bluetooth/l2cap.h>
 #include <sdpd.h>
 #include "../src/dbus-common.h"
 #include <unistd.h>
@@ -260,7 +260,7 @@ static gint cmp_chan_mdl(gconstpointer a, gconstpointer mdl)
 	return -1;
 }
 
-static uint8_t get_app_id()
+static uint8_t get_app_id(void)
 {
 	uint8_t id = next_app_id;
 
@@ -510,14 +510,23 @@ static void hdp_mdl_reconn_cb(struct mcap_mdl *mdl, GError *err, gpointer data)
 	}
 
 	fd = mcap_mdl_get_fd(dc_data->hdp_chann->mdl);
-	if (fd < 0)
+	if (fd < 0) {
 		reply = g_dbus_create_error(dc_data->msg,
 						ERROR_INTERFACE ".HealthError",
 						"Cannot get file descriptor");
+		g_dbus_send_message(dc_data->conn, reply);
+		return;
+	}
 
-	reply = g_dbus_create_reply(dc_data->msg, DBUS_TYPE_UNIX_FD, &fd,
-							DBUS_TYPE_INVALID);
+	reply = g_dbus_create_reply(dc_data->msg, DBUS_TYPE_UNIX_FD,
+							&fd, DBUS_TYPE_INVALID);
 	g_dbus_send_message(dc_data->conn, reply);
+
+	g_dbus_emit_signal(dc_data->conn,
+			device_get_path(dc_data->hdp_chann->dev->dev),
+			HEALTH_DEVICE, "ChannelConnected",
+			DBUS_TYPE_OBJECT_PATH, &dc_data->hdp_chann->path,
+			DBUS_TYPE_INVALID);
 }
 
 static void hdp_get_dcpsm_cb(uint16_t dcpsm, gpointer user_data, GError *err)
@@ -578,7 +587,6 @@ static void device_reconnect_mdl_cb(struct mcap_mdl *mdl, GError *err,
 	g_dbus_send_message(dc_data->conn, reply);
 	hdp_tmp_dc_data_unref(dc_data);
 	g_error_free(gerr);
-	gerr = NULL;
 
 	/* Send abort request because remote side is now in PENDING state */
 	if (!mcap_mdl_abort(mdl, abort_mdl_cb, NULL, NULL, &gerr)) {
@@ -1279,7 +1287,7 @@ static void mcl_uncached(struct mcap_mcl *mcl, gpointer data)
 	DBG("Mcl uncached %s", path);
 }
 
-static void check_devices_mcl()
+static void check_devices_mcl(void)
 {
 	struct hdp_device *dev;
 	GSList *l, *to_delete = NULL;
@@ -1329,7 +1337,7 @@ static gboolean update_adapter(struct hdp_adapter *hdp_adapter)
 		goto update;
 
 	adapter_get_address(hdp_adapter->btd_adapter, &addr);
-	hdp_adapter->mi = mcap_create_instance(&addr, BT_IO_SEC_HIGH, 0, 0,
+	hdp_adapter->mi = mcap_create_instance(&addr, BT_IO_SEC_MEDIUM, 0, 0,
 					mcl_connected, mcl_reconnected,
 					mcl_disconnected, mcl_uncached,
 					NULL, /* CSP is not used by now */
@@ -1462,7 +1470,7 @@ static void destroy_create_dc_data(gpointer data)
 	hdp_create_data_unref(dc_data);
 }
 
-static void *generate_echo_packet()
+static void *generate_echo_packet(void)
 {
 	uint8_t *buf;
 	int i;
@@ -1727,7 +1735,6 @@ static void device_create_mdl_cb(struct mcap_mdl *mdl, uint8_t conf,
 
 	error("%s", gerr->message);
 	g_error_free(gerr);
-	gerr = NULL;
 
 	reply = g_dbus_create_reply(hdp_conn->msg,
 					DBUS_TYPE_OBJECT_PATH, &hdp_chan->path,
@@ -1749,7 +1756,6 @@ fail:
 						"%s", gerr->message);
 	g_dbus_send_message(user_data->conn, reply);
 	g_error_free(gerr);
-	gerr = NULL;
 
 	/* Send abort request because remote side is now in PENDING */
 	/* state. Then we have to delete it because we couldn't */
@@ -2049,7 +2055,7 @@ static DBusMessage *device_get_properties(DBusConnection *conn,
 		path = g_strdup(device->fr->path);
 	else
 		path = g_strdup("");
-	dict_append_entry(&dict, "MainChannel", DBUS_TYPE_STRING, &path);
+	dict_append_entry(&dict, "MainChannel", DBUS_TYPE_OBJECT_PATH, &path);
 	g_free(path);
 	dbus_message_iter_close_container(&iter, &dict);
 
@@ -2184,7 +2190,7 @@ int hdp_manager_start(DBusConnection *conn)
 	return 0;
 }
 
-void hdp_manager_stop()
+void hdp_manager_stop(void)
 {
 	g_dbus_unregister_interface(connection, MANAGER_PATH, HEALTH_MANAGER);
 
